{ ***************************************************}
{                                                    }
{   Delphi Visual Component Library                  }
{   Non-visual TSpeaker component for Delphi & BCB   }
{                                                    }
{   Copyright (C) 2002 - 2005 MacroSoftware          }
{                                                    }
{   All Rights Reserved.                             }
{                                                    }
{ ***************************************************}

unit Speaker;

interface

uses
  Windows, Classes, Forms, DesignIntf;

type
  TSpeakerClass = class of TSpeaker;

  TSoundType = (stNone, stSingle, stMessage, stAlarm, stMonitor, stSequence);

  TSoundEvent = procedure(Sender: TObject; Handled: Boolean;
    SoundType: TSoundType) of object;

  TCounter = 1..MAXSHORT;
  TFrequency = 37..MAXSHORT;

  TSoundMessage = (smInformation, smDone, smWarning, smError, smNo, smWrong,
    smChord, smWhisle, smCall, smOldPhone, smWelcome, smOfficial, smDazzle,
    smFall, smCountdown, smHanging, smClimb, smDash, smDot, smAlarm
  );
  TSoundAlarm = (saStandard, saYellow, saRed, saEmergency);

  TSpeaker = class(TComponent)
  private
    FOnSound: TSoundEvent;
    FSequenced: Boolean;
    FSetDesignMessage: Boolean;
    FSetDesignSound: Boolean;
    FCounter: TCounter;
    FFrequency: TFrequency;
    FDuration: Cardinal;
    FSoundMessage: TSoundMessage;
    FSoundAlarm: TSoundAlarm;
    FDisableAlarm: Boolean;
    FDisableMonitor: Boolean;
    procedure SetDesignMessage(Value: Boolean);
    procedure SetDesignSound(Value: Boolean);
    procedure SetPort(Address, Value: Word);
    function GetPort(Address: Word): Word;
    procedure Sound(Frequency: Word); overload;
    procedure Sound(Frequency: TFrequency; Duration: Cardinal); overload;
    procedure NoSound;
    procedure Delay(const Miliseconds: Cardinal);
    function IsWinNT: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BeginSequence;
    procedure EndSequence;
    procedure PlaySound; overload;
    procedure PlaySound(AFrequency: TFrequency; ADuration: Cardinal); overload;
    procedure PlayMessage; overload;
    procedure PlayMessage(ASoundMessage: TSoundMessage); overload;
    procedure EnableAlarm;
    procedure DisableAlarm;
    procedure EnableMonitor;
    procedure DisableMonitor;
  published
    property Counter: TCounter read FCounter write FCounter
      default 7;
    property DesignMessage: Boolean read FSetDesignMessage write SetDesignMessage
      stored False;
    property DesignSound: Boolean read FSetDesignSound write SetDesignSound
      stored False;
    property Frequency: TFrequency read FFrequency write FFrequency
      default 100;
    property Duration: Cardinal read FDuration write FDuration
      default 1000;
    property SoundMessage: TSoundMessage read FSoundMessage write FSoundMessage
      default  smInformation;
    property SoundAlarm: TSoundAlarm read FSoundAlarm write FSoundAlarm
      default  saStandard;
    property OnSound: TSoundEvent read FOnSound write FOnSound;
  end;

procedure Register;

implementation

constructor TSpeaker.Create(AOwner: TComponent);
begin
  inherited Create(Aowner);
  FSequenced := False;
  FDisableAlarm := False;
  FDisableMonitor := False;
  FSetDesignMessage := False;
  FSetDesignSound := False;
  Counter := 7;
  Frequency := 100;
  Duration := 1000;
  SoundMessage := smInformation;
  SoundAlarm := saStandard;
end;

destructor TSpeaker.Destroy;
begin
  inherited Destroy;
end;

procedure TSpeaker.BeginSequence;
begin
  FSequenced := True;
  if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.EndSequence;
begin
  FSequenced := False;
  if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.SetDesignMessage(Value: Boolean);
begin
  if (csDesigning in ComponentState) and (FSoundMessage = smAlarm) then
  begin
    MessageBox(
      Application.Handle,
      'You can not enable alarm at design-time!' + #13#13 +
      'To enable alarm sequence you must first call EnableAlarm procedure ' + #13 +
      'in your source code and then (re)compile project.',
      'Design-time',
      MB_ICONWARNING
    );
    Exit;
  end else
  if not (csDesigning in ComponentState) then
  begin
    MessageBox(
      Application.Handle,
      'You can not play message at run-time using DesignMessage property!',
      'Run-time',
      MB_ICONERROR
    );
    Exit;
  end else
  begin
    FSetDesignMessage := Value;
    PlayMessage;
    FSetDesignMessage := not Value;
  end;
end;

procedure TSpeaker.SetDesignSound(Value: Boolean);
begin
  if (csDesigning in ComponentState) and (FDuration > 15000) then
  begin
    MessageBox(
      Application.Handle,
      'You can not play sound at design-time if duration is longer than 15 seconds ' +
      '(in other words if Duration property value is greater than 15000)!',
      'Design-time',
      MB_ICONWARNING)
    ;
    Exit;
  end else
  if not (csDesigning in ComponentState) then
  begin
    MessageBox(
      Application.Handle,
      'You can not play message at run-time using DesignSound property!',
      'Run-time',
      MB_ICONERROR
    );
    Exit;
  end else
  begin
    FSetDesignSound := Value;
    Sound(FFrequency, FDuration);
    FSetDesignSound := not Value;
  end;
end;

function TSpeaker.GetPort(Address: Word): Word;
var
  bValue: Byte;
begin
  asm
    mov DX, Address
    in  AL, DX
    mov bValue, AL
  end;
  Result := bValue;
end;

procedure TSpeaker.SetPort(Address, Value: Word);
var
  bValue: Byte;
begin
  bValue := Trunc(Value and 255);
  asm
    MOV DX, Address
    MOV AL, bValue
    OUT DX, AL
  end;
end;

procedure TSpeaker.Sound(Frequency: Word);
var
  B: Byte;
begin
  if Frequency > 18 then
  begin
    Frequency := Word(1193181 div LongInt(Frequency));
    B := GetPort($61);
    if (B and 3) = 0 then
    begin
      SetPort($61, B or 3);
      //SetPort($43, $B6);
    end;
    SetPort($42, Frequency);
    SetPort($42, (Frequency shr 8));
  end;
end;

procedure TSpeaker.NoSound;
var
  wValue: Word;
begin
  wValue := GetPort($61);
  wValue := wValue and $FC;
  SetPort($61, wValue);
  if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.Delay(const Miliseconds: Cardinal);
asm
  push Miliseconds
  call Sleep
end;

function TSpeaker.IsWinNT: Boolean;
var
  OS: TOsVersionInfo;
begin
  OS.dwOSVersionInfoSize := SizeOf(OS);
  GetVersionEx(OS);
  if OS.dwPlatformId = VER_PLATFORM_WIN32_NT then
    Result := True else Result := False;
end;

procedure TSpeaker.Sound(Frequency: TFrequency; Duration: Cardinal);
begin
  if not IsWinNT then
  begin
    Sound(Frequency);
    Delay(Duration);
    NoSound;
  end else Beep(Frequency, Duration);
end;

procedure TSpeaker.PlaySound;
var
  SoundType: TSoundType;
begin
  if FSequenced then SoundType := stSequence else SoundType := stSingle;
  if Assigned(FOnSound) then OnSound(Self, True, SoundType);
  if not IsWinNT then
    Sound(FFrequency, FDuration) else
    Beep(FFrequency, FDuration);
  if not FSequenced then if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.PlaySound(AFrequency: TFrequency; ADuration: Cardinal);
begin
  Frequency := AFrequency;
  Duration := ADuration;
  PlaySound;
end;

procedure TSpeaker.EnableAlarm;
begin
  if Assigned(FOnSound) then OnSound(Self, True, stAlarm);
  FDisableAlarm := False;
  case FSoundAlarm of
    saStandard:
    begin
      repeat
        Sound(2099, 240);
        Delay(130);
        Application.ProcessMessages;
      until FDisableAlarm;
    end;
    saYellow:
    begin
      repeat
        Sound(1000, 300);
        Delay(150);
        Application.ProcessMessages;
      until FDisableAlarm;
    end;
    saRed:
    begin
      repeat
        Sound(1999, 400);
        Delay(100);
        Application.ProcessMessages;
      until FDisableAlarm;
    end;
    saEmergency:
    begin
      repeat
        Sound(698, 200);
        Sound(494, 200);
        Application.ProcessMessages;
      until FDisableAlarm;
    end;
  end;
  if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.DisableAlarm;
begin
  FDisableAlarm := True;
end;

procedure TSpeaker.PlayMessage;
var
  I: Integer;
begin
  if Assigned(FOnSound) then OnSound(Self, True, stMessage);
  case FSoundMessage of
    smInformation:
    begin
      Sound(1047, 100);
      Sound(1109, 100);
      Sound(1175, 100);
    end;
    smDone:
    begin
      Sound(932, 100);
      Sound(988, 100);
      Sound(1046, 100);
    end;
    smWarning:
    begin
      Sound(2093, 100);
      Sound(1976, 100);
      Sound(1857, 100);
    end;
    smError: Sound(40, 500);
    smNo:
    begin
      Sound(92, 150);
      Delay(200);
      Sound(64, 500);
    end;
    smWrong:
    begin
      Sound(261, 150);
      Sound(247, 50);
      Sound(233, 50);
      Sound(220, 50);
      Sound(207, 50);
      Sound(196, 50);
      Sound(189, 50);
      Sound(174, 50);
      Sound(165, 50);
      Sound(155, 50);
      Sound(149, 50);
      Sound(138, 50);
      Sound(131, 100);
      Sound(65, 200);
    end;
    smChord:
    begin
      Sound(494, 80);
      Sound(466, 80);
      Sound(440, 80);
      Sound(415, 80);
      Sound(392, 80);
      Sound(370, 80);
      Sound(349, 80);
      Sound(329, 80);
      Sound(311, 80);
      Sound(293, 80);
      Sound(277, 80);
      Sound(261, 80);
    end;
    smWhisle:
    begin
      Sound(523, 80);
      Sound(698, 80);
      Sound(784, 80);
      Sound(1046, 80);
    end;
    smCall:
    begin
      Sound(196, 650);
      Delay(100);
      Sound(165, 500);
    end;
    smOldPhone:
    begin
      Sound(54, 910);
      Delay(128);
      Sound(54, 910);
    end;
    smWelcome:
    begin
      Sound(932, 100);
      Sound(988, 100);
      Sound(1046, 100);
      Delay(100);
      Sound(1046, 100);
      Sound(988, 100);
      Sound(932, 100);
    end;
    smOfficial:
    begin
      Sound(196, 200);
      Delay(50);
      Sound(196, 200);
      Delay(50);
      Sound(196, 200);
      Delay(50);
      Sound(165, 700);
      Delay(100);
      Sound(261, 200);
      Delay(50);
      Sound(261, 200);
      Delay(50);
      Sound(261, 200);
      Delay(50);
      Sound(131, 700);
    end;
    smDazzle:
    begin
      for I := 1 to FCounter do
      begin
        Sound(329, 100);
        Sound(311, 100);
        Sound(293, 100);
        Sound(311, 100);
      end;
    end;
    smFall:
    begin
      Sound(329, 100);
      Sound(311, 100);
      Sound(293, 100);
      Sound(311, 100);
      Sound(349, 100);
      Sound(329, 100);
      Sound(311, 100);
      Sound(329, 100);
      Sound(370, 100);
      Sound(349, 100);
      Sound(329, 100);
      Sound(349, 100);
    end;
    smCountdown:
    begin
      for I := 1 to FCounter do
      begin
        Sound(1090, 90);
        Delay(910);
      end;
      for I := 1 to 5 do
      begin
        Sound(2090, 80);
        Delay(10);
      end;
    end;
    smHanging:
    begin
      Sound(784, 80);
      Sound(1046, 80);
      Sound(830, 80);
      Sound(1109, 80);
      Sound(880, 80);
      Sound(1174, 80);
      Sound(880, 80);
      Sound(1244, 80);
      Sound(988, 80);
      Sound(1318, 80);
      Sound(1046, 80);
      Sound(1397, 80);
    end;
    smClimb:
    begin
      Sound(261, 80);
      Sound(277, 80);
      Sound(293, 80);
      Sound(311, 80);
      Sound(329, 80);
      Sound(349, 80);
      Sound(370, 80);
      Sound(392, 80);
      Sound(415, 80);
      Sound(440, 80);
      Sound(466, 80);
      Sound(494, 80);
    end;
    smDash: Sound(600, 600);
    smDot: Sound(600, 200);
    smAlarm: EnableAlarm;
  end;
  if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.PlayMessage(ASoundMessage: TSoundMessage);
begin
  SoundMessage := ASoundMessage;
  PlayMessage;
end;

procedure TSpeaker.EnableMonitor;
begin
  if Assigned(FOnSound) then OnSound(Self, True, stMonitor);
  FDisableMonitor := False;
  repeat
    Sound(High(TFrequency) div 3, 100);
    Delay(900);
    Application.ProcessMessages;
  until FDisableMonitor;
  if Assigned(FOnSound) then OnSound(Self, False, stNone);
end;

procedure TSpeaker.DisableMonitor;
begin
  FDisableMonitor := True;
end;

procedure Register;
begin
  RegisterComponents('MacroSoftware', [TSpeaker]);
  RegisterPropertiesInCategory('SpeakerSettings', TSpeakerClass(TSpeaker),
    ['Counter', 'Duration', 'Frequency', 'SoundMessage', 'SoundAlarm']);
  RegisterPropertiesInCategory('DesignTime', TSpeakerClass(TSpeaker),
    ['DesignMessage', 'DesignSound']);
  RegisterPropertiesInCategory('SpeakerEvents', TSpeakerClass(TSpeaker),
    ['OnSound']);
end;

end.

